#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

package require tls

proc getPassword {} {
    return "testing"
}


proc openConnection {channel clientaddr clientport } {
    global State
    global tlssignal
    
    # Wait until the handshake is complete
    fileevent $channel readable \
        [list handshakeHandler $channel $clientaddr]
    vwait tlssignal($channel)
    
    fileevent $channel readable [list readLine $channel]
 
   # Create a safe interpreter
    set State(interp.$channel) [::safe::interpCreate]

    # Link the 'writeLog' procedure in this environment
    # to the 'log' procedure in the safe child interpreter.

    $State(interp.$channel) alias log writeLog

    fconfigure $channel -buffering line
    puts $channel "Ready"
}
 
proc handshakeHandler {channel clientaddr} {
    global tlssignal
    
    if {[eof $channel]} {
      close $channel
      return
    }
     
    set fail [catch {tls::handshake $channel} complete]
    
    if {!$fail && $complete} {
        set tlssignal($channel) ""
    }
}

proc readLine {channel} {
    global State

    # Read a line of data
    set len [gets $channel line] 
   
    # Close if we've recevied an EOF from the client
    if {($len <= 0) & [eof $channel]} {
         close $channel
	 ::safe::interpDelete $State(interp.$channel)
         return
    } 
    
    # Put the data in a safe place
    append State(input.$channel) "$line"

    # And check to see if we've got a complete command

    if {(![string equal "" $State(input.$channel)]) &&
        ([info complete $State(input.$channel)])} {
      #  The processing is done assynchronously (using the after)
      #  to allow the client to stack requests if necessary.
      #  Putting the data in the stack means that we can even stack
      #  requests from a single client if necessary.
   
       processMessage $channel
       set State(input.$channel) "" 
    }  else {
       if {(![string equal "" $State(input.$channel)])} {
          append State(input.$channel) "\n" 
       }
    }  
}

proc processMessage {channel} {
   global State
   
   if {[string match "" [string trim $State(interp.$channel)]]} {
       return
   }
   writeLog "EVAL: $State(interp.$channel) eval $State(input.$channel)"
   set rply [$State(interp.$channel) eval $State(input.$channel)]
   puts $channel "$rply"
   puts $channel "Ready"
}

# Define writeLog

proc writeLog {data} {
  set of [open /tmp/agent.log "a"]
  puts $of $data
  close $of
}

set port			5000
set certDir			./SimpleCA28/certificates
set State(time) 20

set s [tls::socket -server openConnection \
    -cafile $certDir/../rootca.pem \
    -certfile $certDir/../1001.pem \
    -keyfile $certDir/www.noucorp.com.key \
    -request 1 -require 1 -password getPassword \
    $port]

vwait forever

